import {
    Metadata, Http, IDictionary, IEventParam, Loader, dayjs,
} from '@zeedhi/core';
import {
    Button, DateRange, LoadingService, Modal, ModalService, Select, SelectMultiple, Text,
} from '@zeedhi/common';
import {
    IterableCarousel, IterableComponentRender,
} from '@zeedhi/teknisa-components-common';
import { TekIterableComponentRender } from '@zeedhi/teknisa-components-vuetify';
import { Utilities } from '@/util/Utilities';
import { getStoredData } from '@zeedhi/zeedhi-miragejs';

export class ListaTarefasController {
    public modalInformacoes!: Modal;

    public modalVisualizarImagens!: Modal;

    public tarefasController = Loader.getInstance('Fm00001TarefasController');

    public modalFiltrarTarefas!: Modal;

    public nrChecklistAgend = '';

    // ---- variáveis que monta as informações adicionais de uma tarefa
    public contrato = '';

    public local = '';

    public cliente = '';

    public tarefa = '';

    public data = '';

    public funcionario = '';

    public situacao = '';

    public tipoOrdenacao = '';

    public ordenarCards({ component }: IEventParam<Button>) {
        const iterable = Metadata.getInstance<TekIterableComponentRender>('iterable-cards-listagem-tarefas');

        iterable.datasource.clearOrder();
        if (component.name === 'botao-ordenar-tarefas') {
            iterable.datasource.addOrder('NMCHECKLISTAGEND', 'ASC');
        } else if (component.name === 'botao-ordenar-local') {
            iterable.datasource.addOrder('NMLOCALCLIENTE', 'ASC');
        } else if (component.name === 'botao-ordenar-data') {
            iterable.datasource.addOrder('DTHORAAGEND', 'DESC');
        } else {
            iterable.datasource.clearOrder();
        }

        const { order } = iterable.datasource;
        if (order) {
            const campo = (order.length === 0) ? '' : order[0].split('.')[0];
            this.exibirTipoOrdenacao(campo);
        }
        iterable.datasource.setPage(1);
        this.inserirOrdenacao({ component: iterable });

        iterable.datasource.setPage(1);
    }

    public exibirTipoOrdenacao(name: any) {
        const visibilidadeText = Metadata.getInstance<Text>('titulo-ordenacao');

        switch (name) {
            case 'NMCHECKLISTAGEND':
                this.tipoOrdenacao = 'Ordenado por nome da Tarefa';
                visibilidadeText.isVisible = true;
                break;
            case 'NMLOCALCLIENTE':
                this.tipoOrdenacao = 'Ordenado por Local';
                visibilidadeText.isVisible = true;
                break;
            case 'DTHORAAGEND':
                this.tipoOrdenacao = 'Ordenado por Data';
                visibilidadeText.isVisible = true;
                break;
            default:
                this.tipoOrdenacao = '';
                visibilidadeText.isVisible = false;
                break;
        }
    }

    public inserirOrdenacao({ component }: IEventParam<TekIterableComponentRender>) {
        const { order } = component.datasource;
        if (order) {
            Http.post('/insertOrdenacao', {
                NMTELA: 'Lista de Tarefas',
                NMCAMPO: (order.length === 0) ? '' : order[0].split('.')[0],
                IDSENTIDO: (order.length === 0) ? '' : order[0].split('.')[1],
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            });
        }
    }

    public onMountedOrdenacao({ component }: IEventParam<TekIterableComponentRender>) {
        Http.post('/getOrdenacao', {
            NMTELA: 'Lista de Tarefas',
        }).then((response: IDictionary) => {
            if (response.data.length !== 0) {
                component.datasource.clearOrder();
                component.datasource.addOrder(response.data[0].NMCAMPO, response.data[0].IDSENTIDO);
                this.exibirTipoOrdenacao(response.data[0].NMCAMPO);
                Utilities.showSuccessMessage();
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public sortFunction(value1: any, value2: any, columnName: string): boolean {
        if (columnName === 'DTHORAAGEND') {
          return (dayjs(value1, 'DD/MM/YYYY HH24:MI').isAfter(dayjs(value2, 'DD/MM/YYYY HH24:MI')));
        }
        return (value1 > value2);
    }

    public stylizeStatus({ component }: IEventParam<Text>) {
        if (component.text === 'Pendente') {
            component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 10px;'>P E N D E N T E</span>";
        }
        if (component.text === 'Concluído') {
            component.text = "<span  style='background-color: #dee2e6; color: green; font-size: 10px;'>C O N C L U Í D O</span>";
        }
        if (component.text === 'Em andamento') {
            component.text = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
        }
        if (component.text === 'Cancelado') {
            component.text = "<span  style='background-color: #dee2e6; color: black; font-size: 10px;'>C A N C E L A D O</span>";
        }
        if (component.text === 'Atrasado') {
            component.text = "<span  style='background-color: #dee2e6; color: red; font-size: 10px;'>A T R A S A D O</span>";
        }
    }

    public async criarModais() {
        this.modalFiltrarTarefas = await ModalService.createFromJson('fm_00001_000_tarefas/modal/fm_00001_001_filtrar_tarefas', true);
        this.modalInformacoes = await ModalService.createFromJson('fm_00001_000_tarefas/modal/fm_00001_001_dados_tarefa', true);
    }

    public destruirModais() {
        ModalService.delete(this.modalFiltrarTarefas);
        ModalService.delete(this.modalInformacoes);
    }

    public async exibirModalInformacoes({ component }: any) {
        LoadingService.show();
        this.tarefasController.rotaTarefasListadas(component).then((response: IDictionary) => {
            const dados = (response.data.data[0] === undefined) ? response.data.data : response.data.data[0];

            this.contrato = (dados.NRCONTRATOFCT === null) ? 'Não há' : dados.NRCONTRATOFCT;
            this.cliente = dados.NMRAZSOCCLIE;
            this.tarefa = dados.NMCHECKLISTAGEND;
            this.local = `${dados.NMLOCALCLIENTE} - ${dados.NMLOCALPAI}`;
            this.data = `${dados.DTAGEND} - ${dados.HORAAGEND}`;
            this.funcionario = dados.NMOPERADOR;
            this.situacao = dados.SITUACAO;

            this.modalInformacoes.show();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public fecharModalInformacoes() {
        this.modalInformacoes.hide();
    }

    public async exibirModalVisualizarImagens({ component }: any) {
        this.modalVisualizarImagens = await ModalService.createFromJson('fm_00001_000_tarefas/modal/fm_00001_001_visualizar_imagens', true);
        const nr = component.name.split('_')[1];
        this.nrChecklistAgend = nr;
        this.modalVisualizarImagens.show();
        setTimeout(() => {
            const tekIterable = Metadata.getInstance<IterableCarousel>('carousel-imagens-anexo-tarefas');
            this.carregarFotosTarefas({ component: tekIterable });
        }, 300);
    }

    public fecharModalVisualizarImagens() {
        this.modalVisualizarImagens.hide();
        ModalService.delete(this.modalVisualizarImagens);
    }

    public async exibirModalFiltros() {
        this.modalFiltrarTarefas.show();
    }

    public fecharModalFiltros() {
        this.modalFiltrarTarefas.hide();
    }

    public carregarCardsTarefas({ component }: any) {
        component.datasource.setFilter({
            TIPO: 1,
        });
        setTimeout(() => {
            this.onMountedOrdenacao({ component });
        }, 500);
    }

    public filtrarCards() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-cards-listagem-tarefas');

        iterable.datasource.setFilter({
            CDCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-cliente').value,
            CDOPERADOR: Metadata.getInstance<SelectMultiple>('filtro-operador').value,
            IDSITUACAOAGEND: Metadata.getInstance<SelectMultiple>('filtro-situacao').value,
            NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-local').value,
            NRSEQCONTRATO: Metadata.getInstance<SelectMultiple>('filtro-contratos').value,
            DTHORAGEND: Metadata.getInstance<DateRange>('data').value,
            IDATIVO: Metadata.getInstance<Select>('filtro-idativo').value,
            TIPO: 3,
        }).finally(() => {
            this.fecharModalFiltros();
        });
    }

    public carregarFotosTarefas({ component }: any) {
        const nr = this.nrChecklistAgend;
        const btnApagar = Metadata.getInstance<Button>('botao-apagar-imagens');
        LoadingService.show();
        this.rotaCarregarFotosTarefas(nr).then((response: any) => {
            if (response.data.data.length === 0) {
                component.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-tarefas').isVisible = true;
                btnApagar.isVisible = false;
            } else {
                component.iterable.datasource.data = [];
                response.data.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRFOTOTAREFAAGEND: dados.NRFOTOTAREFAAGEND,
                        DSFOTOTAREFAAGEND: dados.DSFOTOTAREFAAGEND,
                    });
                });
                btnApagar.isVisible = true;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public async excluirFoto() {
        const carrosel = Metadata.getInstance<IterableCarousel>('carousel-imagens-anexo-tarefas');
        const img = carrosel.iterable.datasource.data;
        const [deleteData] = img.filter((value: any, index: any) => index === carrosel.currentSlide);

        LoadingService.show();
        this.rotaExcluitFotoTarefas(deleteData).then(() => {
            const data = img.filter((value: any, index: any) => index !== carrosel.currentSlide);
            carrosel.iterable.datasource.updateData(data);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public limparFiltros() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-cards-listagem-tarefas');

        Metadata.getInstance<SelectMultiple>('filtro-cliente').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-operador').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-situacao').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-local').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-contratos').value = null;
        Metadata.getInstance<DateRange>('data').value = null;

        this.carregarCardsTarefas({ component: iterable });
    }

    // FUNÇÕES AUXILIARES PARA CONTROLE DE ROTAS/MIRAGEJS
    public utilizarMirageJs = (window.localStorage.getItem('MIRAGEJS') === 'S');

    public async rotaCarregarFotosTarefas(nr: any) {
        let metodo: any;
        if (!this.utilizarMirageJs) {
            metodo = Http.post('/getFotosTarefas', {
                NRCHECKLISTAGEND: nr,
            });
        } else {
            const fotos = await getStoredData('getFotosTarefas');
            const fotosFiltro: any = fotos?.filter((n) => parseInt(n.NRCHECKLISTAGEND, 10) === parseInt(nr, 10));
            metodo = {
                data: {
                    data: fotosFiltro,
                },
            };
        }
        return metodo;
    }

    public rotaExcluitFotoTarefas(deleteData: any) {
        let metodo: any;
        if (!this.utilizarMirageJs) {
           metodo = Http.post('/excluirFotoTarefas', {
                NRFOTOTAREFAAGEND: deleteData.NRFOTOTAREFAAGEND,
            });
        } else {
            Http.delete(`/getFotosTarefas/${deleteData.NRFOTOTAREFAAGEND}`);
          metodo = Http.post('/excluirFotoOffline', {
                NRFOTOTAREFAAGEND: deleteData.NRFOTOTAREFAAGEND,
            });
        }
        return metodo;
    }
}
